#include <ruby.h>
#include "cornflake.h"

static VALUE rb_cCornflake;

static void
rb_cornflake__free(void *corn)
{
	cornflake_close(corn);
}

static VALUE
rb_cornflake__new(VALUE klass, VALUE rb_path)
{
	struct cornflake *corn;

	Check_Type(rb_path, T_STRING);

	corn = cornflake_new(StringValueCStr(rb_path));
	if (!corn)
		rb_raise(rb_eRuntimeError, "Failed to open Cornflake state at %s",
			StringValueCStr(rb_path));

	return Data_Wrap_Struct(klass, NULL, rb_cornflake__free, corn);
}

static void
rb_cornflake_check(int result)
{
	switch (result) {
	case CORNFLAKE_OK:
		return; 

	case CORNFLAKE_ELOCK:
		rb_raise(rb_eRuntimeError, "Failed to lock the global Cornflake state");
		break;

	case CORNFLAKE_EDRIFT:
		rb_raise(rb_eRuntimeError, "Computer time drifted into the past");
		break;

	default:
		rb_raise(rb_eRuntimeError, "Failed to generate UID");
		break;
	}
}

static VALUE
rb_cornflake_gen(VALUE self)
{
	uint8_t uid[16];
	struct cornflake *corn;

	Data_Get_Struct(self, struct cornflake, corn);
	rb_cornflake_check(cornflake_gen(uid, corn));

	return rb_str_new((char *)uid, 16);
}

static VALUE
rb_cornflake_gen_hex(VALUE self)
{
	char hex_uid[33];
	struct cornflake *corn;

	Data_Get_Struct(self, struct cornflake, corn);
	rb_cornflake_check(cornflake_gen_hex(hex_uid, sizeof(hex_uid), corn));

	return rb_str_new2(hex_uid);
}

static VALUE
rb_cornflake_set_mac(VALUE klass, VALUE rb_mac)
{
	Check_Type(rb_mac, T_STRING);

	if (cornflake_set_mac(StringValueCStr(rb_mac)) < 0)
		rb_raise(rb_eTypeError, "Invalid MAC Address: %s", StringValueCStr(rb_mac));

	return Qnil;
}

void Init_cornflake()
{
	rb_cCornflake = rb_define_class("Cornflake", rb_cObject);
	rb_define_singleton_method(rb_cCornflake, "new", rb_cornflake__new, 1);
	rb_define_singleton_method(rb_cCornflake, "macaddr=", rb_cornflake_set_mac, 1);
	rb_define_method(rb_cCornflake, "uid", rb_cornflake_gen, 0);
	rb_define_method(rb_cCornflake, "hex", rb_cornflake_gen_hex, 0);
}

